#include "main.h"

typedef struct
{
  unsigned int TMR0_TRIGOV;
  unsigned int TMR1_TRIGOV;
  unsigned int EVT_MISS[24];
} ADCC_ERR_INTR_CNT;


unsigned int ADCC_TMR0_FRM_COMP_CNT, ADCC_TMR1_FRM_COMP_CNT;
unsigned char ADCC_TMR0_DONE, ADCC_TMR1_DONE; 
unsigned char ADCC_TMR0_Final_Data, ADCC_TMR1_Final_Data;
ADCC_ERR_INTR_CNT ERR_COUNTER;

/****************************************************************************************************************************************
*	Function name:  Init_Interrupts
*	Description	:	This function enables or disable the ADCC interupt.
*					Both ADCC TImer frame completion interrupt are enabled to clear the mandatory latch bits after frame completion. 
*					These frame completion ISR's also checks whether required number of ADC samples has been received or not.
*					ADCC Error interrupt is also enabled and is enabled and that for ADCC trigger input overrun conditions and event miss condition. 
*					other error interrupts such as event collision or memory/address errors are not enabled.
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void Init_Interrupts(void)
{
  	*pREG_ADCC0_ERRMSK_SET = 0x0000007C;		//	1: disable, 0: enable
	*pREG_ADCC0_EIMSK_SET  = 0x00000000;		//	1: disable, 0: enable
	*pREG_ADCC0_FIMSK_SET  = 0x00000000;		//	1: disable, 0: enable
	
	NVIC_SetPriority(INTR_ADCC0_TMR0_EVT,0);   	// NVIC_SetPriority(IRQn_Type IRQn, uint32_t priority)
	NVIC_EnableIRQ(INTR_ADCC0_TMR0_EVT);		// NVIC_EnableIRQ(IRQn_Type IRQn)	
	NVIC_SetPriority(INTR_ADCC0_TMR1_EVT,0);   	// NVIC_SetPriority(IRQn_Type IRQn, uint32_t priority)
	NVIC_EnableIRQ(INTR_ADCC0_TMR1_EVT);		// NVIC_EnableIRQ(IRQn_Type IRQn)	
	NVIC_SetPriority(INTR_ADCC0_ERR,0);   		// NVIC_SetPriority(IRQn_Type IRQn, uint32_t priority)
	NVIC_EnableIRQ(INTR_ADCC0_ERR);				// NVIC_EnableIRQ(IRQn_Type IRQn)
}



/****************************************************************************************************************************************
*	Function name:  ADCC0_TMR0_EVT_Int_Handler, ADCC0_TMR0_EVT_Int_Handler, ADCC0_ERR_Int_Handler
*	Description	:	When ADCC error condition occurs, this function is called. It checks if the interrupt is dur to trigger overrun condition
					or due to event miss condition. 
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void ADCC0_TMR0_EVT_Int_Handler (void)
{
	*pREG_ADCC0_FISTAT = BITM_ADCC_FISTAT_FINT0 ;
	ADCC_TMR0_FRM_COMP_CNT++;
	
	if(ADCC_TMR0_Final_Data == 1)
	{
		ADCC_TMR0_DONE = 1;
		ADCC_TMR0_Final_Data = 0;
	}
	if(ADCC_TMR0_FRM_COMP_CNT == BUFF_SIZE-1)
	{
	  	*pREG_TIMER0_RUN_CLR  =  (BITM_TIMER_RUN_SET_TMR04) ;
		ADCC_TMR0_Final_Data = 1;
	}
}
//=========================================================================================================

void ADCC0_TMR1_EVT_Int_Handler (void)
{
	*pREG_ADCC0_FISTAT = BITM_ADCC_FISTAT_FINT1 ;
	ADCC_TMR1_FRM_COMP_CNT++;
	
	if(ADCC_TMR1_Final_Data == 1)
	{
		ADCC_TMR1_DONE = 1;
		ADCC_TMR1_Final_Data = 0;
	}
	if(ADCC_TMR1_FRM_COMP_CNT == BUFF_SIZE-1)
	{
		*pREG_TIMER0_RUN_CLR  =  (BITM_TIMER_RUN_SET_TMR05) ;
		ADCC_TMR1_Final_Data = 1;
	}
}
//=========================================================================================================

void ADCC0_ERR_Int_Handler (void)
{
	unsigned int temp_ERR_STAT = *pREG_ADCC0_ERRSTAT ;

	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_TRGOV0)			{
		*pREG_ADCC0_ERRSTAT = BITM_ADCC_ERRSTAT_TRGOV0 ;
		ERR_COUNTER.TMR0_TRIGOV++;							}

	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_TRGOV1)			{
		*pREG_ADCC0_ERRSTAT = BITM_ADCC_ERRSTAT_TRGOV1 ;
		ERR_COUNTER.TMR1_TRIGOV++;							}

	unsigned int temp_EMISS_STAT, EMISS_CNT ;
	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_EMIS)																											{
		for(temp_EMISS_STAT = *pREG_ADCC0_EMISS, EMISS_CNT = 0; temp_EMISS_STAT != 0; temp_EMISS_STAT = (temp_EMISS_STAT >> 1), 	EMISS_CNT++)	{
			if(temp_EMISS_STAT & 1)																												{
				*pREG_ADCC0_EMISS = (1 << EMISS_CNT);
				ERR_COUNTER.EVT_MISS[EMISS_CNT]++;																								}	}	}

	//	error condition has occurred. So, code fixinf may require
	while(1);
}


/****************************************************************************************************************************************
*	Function name:  clear_interrupt_state
*	Description	:	This funciton clears the interryupt counters and is called before start of each process.
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void clear_interrupt_state(void)
{
	ADCC_TMR0_FRM_COMP_CNT = 0; ADCC_TMR1_FRM_COMP_CNT = 0;
	ADCC_TMR0_DONE =0; ADCC_TMR1_DONE = 0;
}
//*********************************************************************************************************
